package model;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.logging.Logger;

import javax.faces.context.FacesContext;
import javax.faces.event.ActionEvent;

import org.ajax4jsf.model.DataVisitor;
import org.ajax4jsf.model.Range;
import org.ajax4jsf.model.SequenceRange;
import org.richfaces.component.UIExtendedDataTable;
import org.richfaces.model.DataProvider;
import org.richfaces.model.ExtendedTableDataModel;
import org.richfaces.model.ExtendedTableDataModifiableModel;
import org.richfaces.model.selection.Selection;
import org.richfaces.model.selection.SimpleSelection;

import util.JSFUtils;

import domain.Trade;
import domain.TradeSearchParams;

public class TradeManagementBean implements DataVisitor {

	private Logger log = Logger.getLogger("model.TradeManagementBean");
	private UIExtendedDataTable table;	
	private String sortMode="multi";
	private String selectionMode="multi";
	private Object tableState;
	private Selection selection = new SimpleSelection();
	private List<Trade> trades = new ArrayList<Trade>();
	private List<Trade> orgTrades = new ArrayList<Trade>();
	private ExtendedTableDataModifiableModel<Trade> dataModel;
	private List<Trade> selectedTrades = new ArrayList<Trade>();
	private Trade currentItem = new Trade();
	private int currentRow;
	private int refreshSeconds = 30;
	private boolean refreshActive = false;
	private boolean keepSearchPanelOpen = true;


	private TradeSearchParams searchParams = new TradeSearchParams();
	
	public TradeManagementBean() {
	
	}
	
	public void fetchCurrentRow(ActionEvent event) {
		String transactionId=(FacesContext.getCurrentInstance().
				getExternalContext().getRequestParameterMap().get("transactionId"));
		currentRow = Integer.parseInt(FacesContext.getCurrentInstance().
				getExternalContext().getRequestParameterMap().get("row"));
		for (Trade item : trades) {
			if (item.getTransactionId().equals(transactionId)){
				currentItem=item;
				break;
			}
		}
	}

	public void search(ActionEvent event)
	{
		log.info("Search");
		
		currentRow = -1;
		currentItem = null;
		((SimpleSelection)selection).clear();
		trades.clear();
		selectedTrades.clear();
		orgTrades = ((TradeBean)JSFUtils.getBean("tradeBean")).getTrades();
		
		for (Trade t : orgTrades)
		{
			if (searchParams.getFromDate() != null)
			{
				if (t.getTransactionTime().compareTo(searchParams.getFromDate()) < 0)
					continue;
			}
			if (searchParams.getToDate() != null)
			{
				if (t.getTransactionTime().compareTo(searchParams.getToDate()) > 0)
					continue;
			}
			if (searchParams.isBuy() || searchParams.isSell())
			{
				boolean buy = t.getSide().equals("Buy");
				boolean sell = t.getSide().equals("Sell");
				if (!((buy && searchParams.isBuy()) || (sell && searchParams.isSell())))
					continue;
					
			}
			if (searchParams.getSecurityTypes() != null && searchParams.getSecurityTypes().size() > 0)
			{
				if (!searchParams.getSecurityTypes().contains(t.getSecurityType()))
					continue;
			}
			if (searchParams.getTidmText() != null && searchParams.getTidmText().length() > 0)
			{
				if (!searchParams.isTIDMMatch(t.getSymbol()))
					continue;
			}
			trades.add(t);
		}
		createDataModel();
	}
	
	public void removeSelected()
	{
		//for ()
	}
	
	public void delete()
	{
		removeTrade(currentItem);		
		currentItem=null;
		currentRow=-1;				
		createDataModel();
	}
	
	private void removeTrade(Trade trade)
	{
		SimpleSelection sSelection = (SimpleSelection) selection;
		
		Object rowKey = dataModel.getKey(trade);
		log.info("removeTrade ["+trade+"] rowKey ["+rowKey+"]");
		
		trades.remove(trade);
		if (sSelection.isSelected(rowKey))
		{			
			sSelection.removeKey(rowKey);
			takeSelection();
		}
		//walkDataModel(false);
		
	}

	private void walkDataModel(Object arg) {
		Range range = new SequenceRange(0, -1);
		try {
			dataModel.walk(FacesContext.getCurrentInstance(), this, range, arg);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			log.severe("Exception: " + e);
		}
	}
	
	public void store()
	{
		log.info("store ["+currentItem+"] rowKey ["+currentRow+"]");
		walkDataModel(false);
		trades.set(currentRow-1, currentItem);
		//walkDataModel(false);
	}
	
	public String saveCurrent()
	{
		store();
		return "success";
	}
	
	
	public void selectAll(ActionEvent event)
	{
		toggleSelectAll(true);
	}
	
	public void deselectAll(ActionEvent event)
	{
		toggleSelectAll(false);
		
		
	}
	
	private void toggleSelectAll(boolean on)
	{
		log.info("toggleSelectAll: " + on);
		SimpleSelection sSelection = (SimpleSelection) selection;
		sSelection.clear();
		if (on)
		{
			walkDataModel(on);
		}
		takeSelection();
	}
	
	public String getSortMode() {
		return sortMode;
	}

	public void setSortMode(String sortMode) {
		this.sortMode = sortMode;
	}

	public String getSelectionMode() {
		return selectionMode;
	}

	public void setSelectionMode(String selectionMode) {
		this.selectionMode = selectionMode;
	}

	public void takeSelection(){
		log.info("takeSelection");
		selectedTrades.clear();
		Iterator<Object> iterator = getSelection().getKeys();
		while (iterator.hasNext()) {
			Object key = iterator.next();
			selectedTrades.add(getTradesDataModel().getObjectByKey(key));
		}
		Collections.sort(selectedTrades);
	}
	
	public ExtendedTableDataModifiableModel<Trade> getTradesDataModel() {
		if (dataModel == null) {
			createDataModel();
		}
		return dataModel;
	}

	private void createDataModel() {
		dataModel = new ExtendedTableDataModifiableModel<Trade>(new DataProvider<Trade>(){

			private static final long serialVersionUID = 5054087821033164847L;

			public Trade getItemByKey(Object key) {
				for(Trade c : trades){
					if (key.equals(getKey(c))){
						return c;
					}
				}
				return null;
			}

			public List<Trade> getItemsByRange(int firstRow, int endRow) {
				return trades.subList(firstRow, endRow);
			}

			public Object getKey(Trade item) {
				return item.getTransactionId();
			}

			public int getRowCount() {
				return trades.size();
			}
			
		});
	}

	public void setTrades(List<Trade> trades) {
		
		
		this.trades = (List<Trade>) ((ArrayList)trades).clone();
	}

	public Object getTableState() {
		return tableState;
	}

	public void setTableState(Object tableState) {
		this.tableState = tableState;
	}

	public Selection getSelection() {
		return selection;
	}

	public void setSelection(Selection selection) {
		this.selection = selection;
	}

	public List<Trade> getSelectedTrades() {
		return selectedTrades;
	}

	public void setSelectedTrades(List<Trade> selectedTrades) {
		this.selectedTrades = selectedTrades;
	}

	public UIExtendedDataTable getTable() {
		return table;
	}

	public void setTable(UIExtendedDataTable table) {
		this.table = table;
	}

	public Trade getCurrentItem() {
		return currentItem;
	}

	public void setCurrentItem(Trade currentItem) {
		log.info("setCurrentItem: " + currentItem);
		this.currentItem = currentItem;
	}

	public int getCurrentRow() {
		return currentRow;
	}

	public void setCurrentRow(int currentRow) {
		log.info("setCurrentRow: " + currentRow);
		this.currentRow = currentRow;
	}

	public int getTradesCount()
	{
		int count = trades== null ? 0 : trades.size();		
		return count;
	}
	
	public int getDisplayedTradesCount()
	{		
		return dataModel== null ? 0 : dataModel.getRowCount();
	}
	
	public int getDisplayedTradesCount1()
	{
		
		return dataModel== null ? 0 : dataModel.getRowCount();
	}
	
	public int getSelectedTradesCount()
	{
		return selectedTrades== null ? 0 : selectedTrades.size();
	}
	
	public Trade getFirstSelectedTrade()
	{
		if (selectedTrades.size() == 0)
		{
			return new Trade();
		}
		else 
		{
			return selectedTrades.get(0);
		}
	}
	
	
	public boolean isSearchFormOpen() {
		log.info("isSearchFormOpen: " + (keepSearchPanelOpen || getTradesCount() == 0));
		return keepSearchPanelOpen || (getTradesCount() == 0);
	}

	public boolean isKeepSearchPanelOpen() {
		log.info("isKeepSearchPanelOpen: " + keepSearchPanelOpen);
		return keepSearchPanelOpen;
	}

	public void setKeepSearchPanelOpen(boolean keepSearchPanelOpen) {
		log.info("setKeepSearchPanelOpen: " + keepSearchPanelOpen);
		this.keepSearchPanelOpen = keepSearchPanelOpen;
	}

	
	public int getRefreshSeconds() {
		return refreshSeconds;
	}

	public void setRefreshSeconds(int refreshSeconds) {
		this.refreshSeconds = refreshSeconds;
	}

	public boolean isRefreshActive() {
		return refreshActive;
	}

	public void setRefreshActive(boolean refreshActive) {
		this.refreshActive = refreshActive;
	}

	public TradeSearchParams getSearchParams() {
		return searchParams;
	}

	@Override
	public void process(FacesContext context, Object rowKey, Object arg)
			throws IOException {

		Trade thisTrade = dataModel.getObjectByKey(rowKey);
		log.info("Walk: rowKey ["+rowKey+"] trade ["+thisTrade+"] arg: " + arg);
		SimpleSelection sSelection = (SimpleSelection) selection;
		if (arg instanceof Boolean)
		{
			if (((Boolean)arg).booleanValue()) 		
				sSelection.addKey(rowKey);
		}
		
		
		

	}
	
	
	

}
